@@ -3,12 +3,12 @@ package VCS::CMSynergy;
 # Copyright (c) 2001-2015 argumentum GmbH
 # See COPYRIGHT section in VCS/CMSynergy.pod for usage and distribution rights.
 
-our $VERSION = '1.49';
-
 use 5.008_001;                          # i.e. v5.8.1
 use strict;
 use warnings;
 
+our $VERSION = '1.50';
+
 use VCS::CMSynergy::Client qw(
     is_win32 _fullwin32path $Error $Ccm_command _error);
 
@@ -332,7 +332,7 @@ sub DESTROY
     # is called. Any eval block encountered during processing of DESTROY()
     # will reset $@  - even if no excpetion is thrown. Hence $@
     # might be empty at "print...". 
-    # We localize $@ to avoid this unexpected behaviour.
+    # We localize $@ to avoid this unexpected behavior.
     # FIXME: might be more correct to push localization into the
     # offending methods.
     local $@;
@@ -845,6 +845,31 @@ sub _flatten_rows
     return $rows;
 }
 
+# helper method used by several "ccm foo -show" implementations
+sub _generic_show
+{
+    my ($self, $cmd, $keywords, $row_type) = @_;
+
+    my $want = _want($row_type, $keywords);
+    my $format = $RS . join($FS, values %$want) . $FS;
+
+    my ($rc, $out, $err) = $self->ccm( 
+            @$cmd, qw/-u -ns -nch -nf/, -format => $format);
+    return $self->set_error($err || $out) unless $rc == 0;
+
+    # split $out at $RS and ignore the first element
+    # (which is either empty or a header like "Baseline...:")
+    my (undef, @records) = split(/\Q$RS\E/, $out);
+
+    my @result;
+    foreach (@records)
+    {
+        my @cols = split(/\Q$FS\E/, $_, -1);    # don't strip empty trailing fields
+        my $row = $self->_query_result($want, \@cols, $row_type);
+        push @result, $row;
+    }
+    return \@result;
+}
 
 sub history
 {
@@ -992,130 +1017,6 @@ sub _history_classic
     return \@result;
 }
 
-# Consider the following history
-# 
-#   ...  ->  foo,3  ->  foo,4  ->  foo,5  ->  bar,6  ->  bar,7  ->  ...
-# 
-# The "history" command reports only a partial view of that.
-# E.g. invoked as "ccm history foo,N:ascii:1" for some N it shows only the chain
-# of versions of "foo" plus "bar,6". predecessor/successor lists are 
-# artificially cut off at the "rename points",
-# so neither "ccm history foo,1:ascii:1" nor "ccm history bar,6:ascii:1" has the
-# complete predecessor/successor information. 
-# 
-# In order to produce a complete history it is _not_ enough to simply
-# "glue" together the graphs for foo,N and bar,N. We must form the
-# "union" of predecessor/successor lists for the overlapping elements.
-# 
-#   ccm history foo,5:ascii:1
-# 
-#   foo,5:ascii:1
-#   Predecessors:
-#           foo,4:ascii:1
-#   Successors:
-#           bar,6:ascii:1
-#   ***************************************************************
-#   bar,6:ascii:1
-#   Predecessors:
-#           foo,5:ascii:1
-#   Successors:
-#   ***************************************************************
-# 
-#   ccm history bar,6:ascii:1
-# 
-#   foo,5:ascii:1
-#   Predecessors:
-#   Successors:
-#           bar,6:ascii:1
-#   ***************************************************************
-#   bar,6:ascii:1
-#   Predecessors:
-#           foo,5:ascii:1
-#   Successors:
-#           bar,7:ascii:1
-#   ***************************************************************
-
-
-# FIXME needs pod
-sub full_history_arrayref
-{
-    my $self = shift;
-    my ($file_spec, $keywords) = validate(\@_, _FILE_SPEC, _KEYWORDS);
-
-    return _flatten_rows($self->_full_history($file_spec, $keywords), $keywords);
-}
-
-# FIXME needs pod
-sub full_history_hashref
-{
-    my $self = shift;
-    my ($file_spec, $keywords) = validate(\@_, _FILE_SPEC, _KEYWORDS);
-
-    return $self->_full_history($file_spec, $keywords);
-}
-
-sub _full_history
-{
-    my ($self, $file_spec, $keywords, $row_type) = @_;
-
-    croak(__PACKAGE__."::full_history_{arrayref,hashref} are not available in web mode")
-        if $self->web_mode;
-
-    # convert $file_spec into a VCS::CMSynergy::Object if it's not already one
-    $file_spec = $self->property(object => $file_spec) or return
-        unless ref $file_spec;
-
-    # augment $keywords with ones needed by the algorithm below
-    my @kw = (@$keywords, qw( object predecessors successors ));
-
-    # keep track of all objects encountered in a hash indexed by objectname;
-    # value is a hash with keys @kw and a count (_seen) how many times 
-    # we've encountered this particular object
-    my %history;
-
-    # keep track of "direct" lineage encountered so far
-    my %nit_seen;      
-
-    my @todo = ($file_spec);
-    while (@todo)
-    {
-        my $obj = pop @todo;
-        $nit_seen{$obj->_nit} = 1;
-
-        foreach my $row (@{ $self->_history($obj, \@kw) })
-        {
-            $obj = $row->{object};
-            push @todo, $obj unless $nit_seen{$obj->_nit};
-
-            if (my $h = $history{$obj})
-            {
-                # we've encountered $row->{object} before: append predecessor
-                # and successor lists of $row to $h ($row and $h should
-                # be identical in all other fields); don't worry about
-                # duplicates in these list, we'll deal with them later
-                push @{ $h->{$_} }, @{ $row->{$_} }
-                    foreach qw( predecessors successors );
-            }
-            else
-            {
-                $history{$obj} = $row;
-            }
-            $history{$obj}->{_seen}++;
-        }
-    }
-
-    my @rows = values %history;
-    foreach my $h (@rows)
-    {
-        if (delete $h->{_seen} > 1)
-        {
-            # make predecessor and successor lists unique
-            $h->{$_} = _uniq_objects($h->{$_})
-                foreach qw( predecessors successors );
-        }
-    }
-    return \@rows;
-}
 
 # not a method
 # returns a new list of objects with duplicates removed 
@@ -1140,7 +1041,7 @@ sub finduse
     # the given objects do not exist (and exits with status 1 unless at least
     # one exists). But for `ccm finduse -query ...', if there are no hits, 
     # the command exits with an error status and produces no output on either 
-    # stdout and stderr. (This is the same behaviour as for `ccm query ...'.) 
+    # stdout and stderr. (This is the same behavior as for `ccm query ...'.) 
     # We will not produce an error in any case. However, the returned array
     # will contain undef in postions corresponding to non-existing objects.
     return [ ] if $rc != 0 and $out eq "" and $err eq "";
@@ -1690,15 +1591,27 @@ sub cat_object
 
     my $want_return = @_ == 1;
     my ($file_spec, $destination) = 
-        validate(\@_, _FILE_SPEC, Optional[Str | GlobRef | FileHandle | ScalarRef | ArrayRef | CodeRef]);
+        validate(\@_, _FILE_SPEC, Optional[Str | GlobRef | FileHandle | ScalarRef]);
 
     my $out;
     $destination = \$out if $want_return;
 
     my ($rc, undef, $err) = $self->_ccm(
-        cat => $file_spec, { out => $destination, binmode_stdout => 1 });
+        cat => $file_spec, 
+        { out => $destination, binmode_stdout => ':raw' });
 
-    return $self->set_error($err || "`ccm cat $file_spec' failed") unless $rc == 0;
+    if (get_logger()->is_trace)
+    {
+        TRACE 
+            ref $destination eq ""
+            ? sprintf("-> out written to file \"%s\"", $destination) :
+            ref $destination eq "SCALAR"
+            ? sprintf("-> out captured to string (%d bytes)", length($$destination)) 
+            : "-> out redirected to filehandle";
+    }
+
+    return $self->set_error($err || "`ccm cat $file_spec' failed") 
+        unless $rc == 0;
     return $want_return ? $out : 1;
 }
 
@@ -1887,7 +1800,7 @@ sub _text_to_tempfile
     my $fh;
     if ($self->{_tempfile})
     {
-        open $fh, "> $self->{_tempfile}"
+        open $fh, ">", $self->{_tempfile}
             or return $self->set_error(qq[can't open temp file "$self->{_tempfile}": $!]); #'
     }
     else
@@ -1910,7 +1823,7 @@ sub _text_to_tempfile
 sub ccm_with_text_editor
 {
     my $self = shift;
-    my ($text, $args) = valiedate(\@_, Str, slurpy ArrayRef);
+    my ($text, $args) = validate(\@_, Str, slurpy ArrayRef);
 
     my $tempfile = $self->_text_to_tempfile($text) or return;
 
@@ -1974,6 +1887,13 @@ sub _projspec2objectname
     return $project;
 }
 
+sub _must_be_one_of
+{
+    my ($what, @choices) = @_;
+    croak(qq["$what" must be one of ].join(", ", @choices))
+        unless grep { $_ eq $what } @choices;
+}
+
 
 # generic wrapper for undefined method "foo":
 #       $ccm->foo(@args)
@@ -1997,15 +1917,7 @@ sub AUTOLOAD
 
     # create the new method on the fly
     no strict 'refs';
-    *{$method} = sub 
-    {
-        my $self = shift;
-
-        my ($rc, $out, $err) = $self->_ccm($method, @_);
-
-        return wantarray ? ($rc, $out, $err) : 1 if $rc == 0;
-        return $self->set_error($err || $out, undef, 0, $rc, $out, $err);
-    };
+    *{$method} = sub { return shift->ccm($method, @_) };
 
     # call it w/o pushing a new stack frame (with same parameters)
     goto &$method;
@@ -2045,49 +1957,63 @@ sub cs_admin    { $_[0]->object("cs:1:admin:1"); }
 sub cvtype      { $_[0]->object("$_[1]:1:cvtype:base"); }
 sub attype      { $_[0]->object("$_[1]:1:attype:base"); }
 
-# FIXME: instead of implementing the inverse function to the
-# ACcent method "displayname" of folder/task/problem objects, one could use
-#    $self>query_object("query_function('$displayname')");
-# but query functions like folder() didn't appear before Synergy 6.x;
-sub _displayname2object
+
+# do a query_object() and expect exactly one result, error out otherwise
+sub _query_object_unique
 {
-    my ($self, $name, $cvtype, $format, $subsys) = @_;
+    my $self = shift;
 
-    # displayname is either <number> (for a local object)
-    # or <dbid><dcm_delimiter><number> (for a foreign object)
-    if ($self->dcm_enabled)
-    {
-        $self->{dcm_prefix_rx} ||= do { my $rx = quotemeta($self->dcm_delimiter); qr/$rx/; };
-        my @parts = split($self->{dcm_prefix_rx}, $name);
-        if (@parts == 2)        { ($subsys, $name) = @parts; }
-        else                    { $subsys = $self->dcm_database_id; }
-    }
+    my $objs = $self->query_object(@_);
+    return $objs->[0] if @$objs == 1;
 
-    return $self->object(sprintf($format, $name), "1", $cvtype, $subsys);
+    my $query = $self->_expand_query($_[0]);
+    return $self->set_error(@$objs == 0
+        ? qq[no object matches query "$query"]
+        : qq[multiple objects match query "$query": @$objs]);
 }
 
-# get folder/task/problem/... object from displayname (without querying Synergy)
-sub folder_object                               # folder('id')
-{ 
-    $_[0]->_displayname2object($_[1], qw/folder %s probtrac/); 
+
+# get folder/task/... object from displayname
+sub baseline_object
+{
+    my $self = shift;
+    my ($baseline_name, @keywords) = @_;
+    return $self->_query_object_unique("baseline('$baseline_name')", @keywords);
 }
-sub task_object                                 # task('id')
-{ 
-    $_[0]->_displayname2object($_[1], qw/task task%s probtrac/); 
-}       
-sub cr_object                                   # cr('id')
-{ 
-    $_[0]->_displayname2object($_[1], qw/problem problem%s probtrac/); 
-}       
-sub baseline_object                             # baseline('id')
-{ 
-    $_[0]->_displayname2object($_[1], qw/baseline %s 1/); 
-}       
+
+sub cr_object
+{
+    my $self = shift;
+    my ($cr_id, @keywords) = @_;
+    return $self->_query_object_unique("cr('$cr_id)'", @keywords);
+}
+
+sub folder_object
+{
+    my $self = shift;
+    my ($folder_id, @keywords) = @_;
+    return $self->_query_object_unique("folder('$folder_id')", @keywords);
+}
+
 sub project_object
 {
     $_[0]->object($_[0]->_projspec2objectname($_[1]));
 }
 
+sub task_object
+{
+    my $self = shift;
+    my ($task_id, @keywords) = @_;
+    return $self->_query_object_unique("task('$task_id')", @keywords);
+}
+
+sub tset_object
+{
+    my $self = shift;
+    my ($tset_name, @keywords) = @_;
+    return $self->_query_object_unique([ type => "tset", tset_name => $tset_name ], @keywords);
+}
+
 
 # $ccm->object_other_version(object, version) => VCS::CMSynergy::Object
 #       new Object with same name/cvtype/instance as OBJECT, but version VERSION
