@@ -19,7 +19,7 @@ with additional methods for Synergy projects.
   $ccm = VCS::CMSynergy->new(%attr);
   ...
   $proj = $ccm->object("editor-1:project:1");
-  print ref $proj;			# "VCS::CMSynergy::Project"
+  print ref $proj;                      # "VCS::CMSynergy::Project"
 
   $proj->chdir_into_wa;
 
@@ -28,7 +28,7 @@ with additional methods for Synergy projects.
 
 This synopsis only lists the major methods.
 
-=cut 
+=cut
 
 use base qw(VCS::CMSynergy::Object);
 
@@ -39,6 +39,10 @@ use Types::Standard qw( Str Optional InstanceOf Maybe
 use File::Spec;
 use Cwd;
 
+# don't blame errors from _must_be_one_of below on one of these
+use vars qw(@ISA);
+our @CARP_NOT = ("VCS::CMSynergy", @ISA);
+
 
 =head1 WORKAREA METHODS
 
@@ -47,7 +51,7 @@ use Cwd;
   my $old_pwd = $proj->chdir_into_wa;
 
 Changes into the toplevel workarea directory of project C<$proj>.
-Returns C<undef> if C<$proj> doesn't maintain a workarea or 
+Returns C<undef> if C<$proj> doesn't maintain a workarea or
 the C<chdir()> failed, otherwise returns the name of current working
 directory before the call.
 
@@ -58,12 +62,12 @@ sub chdir_into_wa
 {
     my $self = shift;
     return $self->ccm->set_error("project `$self' doesn't maintain a workarea")
-	unless $self->get_attribute("maintain_wa") eq "TRUE";
+        unless $self->get_attribute("maintain_wa") eq "TRUE";
 
     my $wa_top = File::Spec->catfile($self->get_attribute("wa_path"), $self->name);
     my $old_pwd = cwd();
-    chdir($wa_top) 
-	or return $self->ccm->set_error("can't chdir($wa_top) into workarea of project `$self': $!");
+    chdir($wa_top)
+        or return $self->ccm->set_error("can't chdir($wa_top) into workarea of project `$self': $!");
     return $old_pwd;
 }
 
@@ -76,27 +80,27 @@ sub chdir_into_wa
   $proj->traverse(\%options, $dir);
 
 C<traverse> walks the tree below directory C<$dir>
-in the invocant project without the need for a workarea. 
+in the invocant project without the need for a workarea.
 It is modelled on L<File::Find>.
 
-C<&wanted> is a code reference described in 
+C<&wanted> is a code reference described in
 L</"wanted function"> below. C<$dir>
 must be a C<VCS::CMSynergy::Object>. If C<$dir> is omitted,
 it defaults to the top level directory of the invocant.
 
 =head3 wanted function
 
-C<&wanted> is called once for all objects below C<$dir> 
+C<&wanted> is called once for all objects below C<$dir>
 including C<$dir> itself. It will also be called on subprojects
 of the incocant project, but C<traverse> will not recurse into
-subprojects unless the C<subprojects> flag is specified 
+subprojects unless the C<subprojects> flag is specified
 (see L</"options"> below).
 
-On each call to C<&wanted>, C<$_> will be bound to the 
-currently traversed object (a C<VCS::CMSynergy::Object>). 
+On each call to C<&wanted>, C<$_> will be bound to the
+currently traversed object (a C<VCS::CMSynergy::Object>).
 
-C<@VCS::CMSynergy::Traversal::dirs> will be bound to 
-an array of C<VCS::CMSynergy::Object>s of cvtype C<dir> representing 
+C<@VCS::CMSynergy::Traversal::dirs> will be bound to
+an array of C<VCS::CMSynergy::Object>s of cvtype C<dir> representing
 the path  from C<$dir> to C<$_> (in the context of the invocant project).
 In particular, C<@VCS::CMSynergy::Traversal::dirs[-1]>
 is the parent C<dir> of C<$_>.
@@ -104,7 +108,7 @@ is the parent C<dir> of C<$_>.
 The convenience function C<VCS::CMSynergy::Traversal::path()>
 returns the filesystem path for C<$_>. It is short for
 
-  join($pathsep, map { $_->name } @VCS::CMSynergy::Traversal::dirs, $_) 
+  join($pathsep, map { $_->name } @VCS::CMSynergy::Traversal::dirs, $_)
 
 where C<$pathsep> is your platform's path separator.
 
@@ -117,13 +121,13 @@ Similarly C<@VCS::CMSynergy::Traversal::projects> represents the
 subproject hierarchy starting with the invocant project.
 In particular, C<$_> is a member of C<$VCS::CMSynergy::Traversal::projects[-1]>.
 
-Note: C<@VCS::CMSynergy::Traversal::dirs> and 
+Note: C<@VCS::CMSynergy::Traversal::dirs> and
 C<@VCS::CMSynergy::Traversal::projects> are both readonly arrays,
 i.e. you can't modify them in any way.
 
 You may set C<$VCS::CMSynergy::Traversal::prune> to a true
 value in C<&wanted> to stop recursion into sub directories (or subprojects)
-(this makes only sense when C<&wanted> is called 
+(this makes only sense when C<&wanted> is called
 on a C<dir> or C<project> object).
 
 If recursion into subprojects is specfied, C<&wanted>
@@ -145,7 +149,7 @@ L</"wanted function">.
 =item C<bydepth> (boolean)
 
 If this option is set, C<traverse>
-calls C<&wanted> on a directory (or project) only B<after> 
+calls C<&wanted> on a directory (or project) only B<after>
 all its entries have been processed. It is "off" by default.
 
 =item C<preprocess> (code reference)
@@ -203,7 +207,7 @@ Note that for any particular C<dir> (or C<project>) object,
 the above code references are always called in order
 C<preprocess>, C<wanted>, C<postprocess>.
 
-Example: 
+Example:
 
   my $proj = $ccm->object('toolkit-1.0:project:1');
 
@@ -226,15 +230,15 @@ Another example:
   $proj->traverse(
     {
       wanted => sub {
-	return unless $_->cvtype eq "project";
-	my $proj_depth = @VCS::CMSynergy::Traversal::projects;
-	print "  " x $proj_depth, $_->displayname, "\n";
+        return unless $_->cvtype eq "project";
+        my $proj_depth = @VCS::CMSynergy::Traversal::projects;
+        print "  " x $proj_depth, $_->displayname, "\n";
       },
       preprocess => sub { sort { $a->name cmp $b->name } @_; },
       subprojects => 1,
     });
 
-This prints the complete project hierarchy rooted at  
+This prints the complete project hierarchy rooted at
 B<toolkit-1.0:project:1>.  Only projects will be shown,
 entries are sorted by name and are intended according to their depth:
 
@@ -246,20 +250,20 @@ entries are sorted by name and are intended according to their depth:
 =cut
 
 # tied array class that acts as a readonly front to a real array
-# NOTE: TIEARRAY expects as first parameter a closure that 
-# returns a reference to the "back" array. Storing the array reference 
+# NOTE: TIEARRAY expects as first parameter a closure that
+# returns a reference to the "back" array. Storing the array reference
 # itself in the tied arraay doesn't work when the "back" array is local'ized.
 {
-    package Tie::ReadonlyArray;	
+    package Tie::ReadonlyArray; 
 
     use Carp;
 
-    sub TIEARRAY	{ bless $_[1], $_[0]; }
-    sub FETCH		{ $_[0]->()->[$_[1]]; }
-    sub FETCHSIZE	{ scalar @{$_[0]->()}; }
+    sub TIEARRAY        { bless $_[1], $_[0]; }
+    sub FETCH           { $_[0]->()->[$_[1]]; }
+    sub FETCHSIZE       { scalar @{$_[0]->()}; }
     *STORE = *STORESIZE = *EXTEND = *CLEAR = *UNTIE
-	= *PUSH = *POP = *UNSHIFT = *SHIFT = *SPLICE 
-	= sub { croak "attempt to modify a readonly array"; };
+        = *PUSH = *POP = *UNSHIFT = *SHIFT = *SPLICE
+        = sub { croak "attempt to modify a readonly array"; };
 }
 
 
@@ -271,66 +275,66 @@ entries are sorted by name and are intended according to their depth:
     our (@_dirs, @_projects, $_pathsep, $_catdirs);
 
     # public
-    our (@dirs, @projects, $prune);		
-    tie @dirs,	   "Tie::ReadonlyArray" => sub { \@_dirs };
+    our (@dirs, @projects, $prune);             
+    tie @dirs,     "Tie::ReadonlyArray" => sub { \@_dirs };
     tie @projects, "Tie::ReadonlyArray" => sub { \@_projects };
 
     # NOTE:references $_ (the currently traversed object)
-    sub path		{ return @_dirs ? 
-			    $_catdirs.$_pathsep.$_->name : $_->name }
+    sub path            { return @_dirs ?
+                            $_catdirs.$_pathsep.$_->name : $_->name }
 
-    sub depth		{ return scalar @_dirs }
+    sub depth           { return scalar @_dirs }
 
-    sub _catdirs	{ $_catdirs = join($_pathsep, map { $_->name } @_dirs) }
+    sub _catdirs        { $_catdirs = join($_pathsep, map { $_->name } @_dirs) }
 }
 
 
 my %traverse_opts =
 (
-    wanted	=> "CODE",
-    preprocess	=> "CODE",
-    postprocess	=> "CODE",
-    attributes	=> "ARRAY",
-    bydepth	=> undef,
-    subprojects	=> undef,
-    pathsep	=> undef,
+    wanted      => "CODE",
+    preprocess  => "CODE",
+    postprocess => "CODE",
+    attributes  => "ARRAY",
+    bydepth     => undef,
+    subprojects => undef,
+    pathsep     => undef,
 );
 
 sub traverse
 {
     my $self = shift;
-    my ($arg_wanted, $dir) = 
+    my ($arg_wanted, $dir) =
         validate(\@_, (CodeRef | HashRef), Optional[InstanceOf["VCS::CMSynergy::Object"]]);
 
     my %wanted;
     if (ref $arg_wanted eq 'CODE')
     {
-	%wanted = ( wanted => $arg_wanted );
+        %wanted = ( wanted => $arg_wanted );
     }
     elsif (ref $arg_wanted eq 'HASH')
     {
-	%wanted = %$arg_wanted;		# make a copy, so we can't inadvertently modify it
-	while (my ($opt, $value) = each %wanted)
-	{
-	    croak(__PACKAGE__.qq[::traverse: argument 1 ("wanted"): unrecognized option "$opt"])
-		unless exists $traverse_opts{$opt};
+        %wanted = %$arg_wanted;         # make a copy, so we can't inadvertently modify it
+        while (my ($opt, $value) = each %wanted)
+        {
+            croak(__PACKAGE__.qq[::traverse: argument 1 ("wanted"): unrecognized option "$opt"])
+                unless exists $traverse_opts{$opt};
 
-	    my $type = $traverse_opts{$opt} or next;
-	    croak(__PACKAGE__.qq[::traverse: argument 1 ("wanted"): option "$opt" must be a $type: $value])
-		unless UNIVERSAL::isa($value, $type);
-	}
-	croak(__PACKAGE__."::traverse: argument 1 (wanted hash ref): option `wanted' is mandatory")
-	    unless $wanted{wanted};
+            my $type = $traverse_opts{$opt} or next;
+            croak(__PACKAGE__.qq[::traverse: argument 1 ("wanted"): option "$opt" must be a $type: $value])
+                unless UNIVERSAL::isa($value, $type);
+        }
+        croak(__PACKAGE__."::traverse: argument 1 (wanted hash ref): option `wanted' is mandatory")
+            unless $wanted{wanted};
     }
 
     if (defined $dir)
     {
-	croak(__PACKAGE__."::traverse: argument 2 (dir) must have cvtype `dir': $dir")
-	    unless $dir->is_dir;
+        croak(__PACKAGE__."::traverse: argument 2 (dir) must have cvtype `dir': $dir")
+            unless $dir->is_dir;
 
-	# check that $dir is indeed a member of $self
+        # check that $dir is indeed a member of $self
         my $parents = $self->has_child($dir);
-	return $self->ccm->set_error("directory `$dir' isn't a member of `$self'")
+        return $self->ccm->set_error("directory `$dir' isn't a member of `$self'")
             unless @$parents;
 
         # fetch its $wanted{attributes}
@@ -338,13 +342,13 @@ sub traverse
     }
     else
     {
-	$dir = $self;
+        $dir = $self;
     }
 
     local @VCS::CMSynergy::Traversal::_projects = ($self);
-    local @VCS::CMSynergy::Traversal::_dirs = (); 
+    local @VCS::CMSynergy::Traversal::_dirs = ();
     local $VCS::CMSynergy::Traversal::_pathsep =
-	(delete $wanted{pathsep}) || VCS::CMSynergy::Client::_pathsep;
+        (delete $wanted{pathsep}) || VCS::CMSynergy::Client::_pathsep();
 
     $self->_traverse(\%wanted, $dir);
 }
@@ -356,62 +360,62 @@ sub _traverse
 
     # NOTE: $parent is either a "dir" or "project" by construction
     my $children = $self->is_child_of(
-	$parent->is_dir ? $parent : undef, @{ $wanted->{attributes} }) 
-	or return;
+        $parent->is_dir ? $parent : undef, @{ $wanted->{attributes} })
+        or return;
 
 
     if ($wanted->{preprocess})
     {
-        # make $_ the current dir/project during preprocess'ing 
-	local $_ = $parent;
-	{ $children = [ $wanted->{preprocess}->(@$children) ]; }
+        # make $_ the current dir/project during preprocess'ing
+        local $_ = $parent;
+        { $children = [ $wanted->{preprocess}->(@$children) ]; }
     }
 
-    if (!$wanted->{bydepth}) 
+    if (!$wanted->{bydepth})
     {
-	local $_ = $parent;
-	local $VCS::CMSynergy::Traversal::prune = 0;
-	{ $wanted->{wanted}->(); }		# protect against wild "next"
-	return 1 if $VCS::CMSynergy::Traversal::prune;
+        local $_ = $parent;
+        local $VCS::CMSynergy::Traversal::prune = 0;
+        { $wanted->{wanted}->(); }              # protect against wild "next"
+        return 1 if $VCS::CMSynergy::Traversal::prune;
     }
 
     push @VCS::CMSynergy::Traversal::_dirs, $parent unless $parent->is_project;
     VCS::CMSynergy::Traversal::_catdirs();
 
-    foreach (@$children)			# localizes $_
+    foreach (@$children)                        # localizes $_
     {
-	if ($_->is_project && $wanted->{subprojects})
-	{
-	    push @VCS::CMSynergy::Traversal::_projects, $_;
-	    $_->_traverse($wanted, $_) or return;
-	    pop @VCS::CMSynergy::Traversal::_projects;
-	    next;
-	}
-	if ($_->is_dir)
-	{
-	    $self->_traverse($wanted, $_) or return;
-	    next;
-	}
+        if ($_->is_project && $wanted->{subprojects})
+        {
+            push @VCS::CMSynergy::Traversal::_projects, $_;
+            $_->_traverse($wanted, $_) or return;
+            pop @VCS::CMSynergy::Traversal::_projects;
+            next;
+        }
+        if ($_->is_dir)
+        {
+            $self->_traverse($wanted, $_) or return;
+            next;
+        }
 
-	{ $wanted->{wanted}->(); }
+        { $wanted->{wanted}->(); }
     }
 
     pop @VCS::CMSynergy::Traversal::_dirs unless $parent->is_project;
     VCS::CMSynergy::Traversal::_catdirs();
-    
-    if ($wanted->{bydepth}) 
+
+    if ($wanted->{bydepth})
     {
         local $_ = $parent;
         local $VCS::CMSynergy::Traversal::prune = 0;
         { $wanted->{wanted}->(); }
-	return 1 if $VCS::CMSynergy::Traversal::prune;
+        return 1 if $VCS::CMSynergy::Traversal::prune;
     }
 
     if ($wanted->{postprocess})
     {
-        # make $_ the current dir/project during postprocess'ing 
-	local $_ = $parent;
-	{ $wanted->{postprocess}->(); }
+        # make $_ the current dir/project during postprocess'ing
+        local $_ = $parent;
+        { $wanted->{postprocess}->(); }
     }
 
     return 1;
@@ -465,7 +469,7 @@ sub hierarchy_project_members
 
 These are convenience methods to enumerate all members of a directory
 (C<is_child_of>) or all directories that contain the object (C<has_child>),
-both in the context of the invocant project 
+both in the context of the invocant project
 
   $members = $proj->is_child_of($dir, @keywords);
 
@@ -495,16 +499,16 @@ to L<VCS::CMSynergy/query_object> as additional keywords.
 sub is_child_of
 {
     my $self = shift;
-    my ($dir, $keywords) = 
+    my ($dir, $keywords) =
         validate(\@_, Maybe[InstanceOf["VCS::CMSynergy::Object"]], VCS::CMSynergy::_KEYWORDS());
     if (defined $dir)
     {
-	croak(__PACKAGE__."::is_child_of: argument 1 ($dir) must have cvtype `dir'")
-	    unless $dir->is_dir;
+        croak(__PACKAGE__."::is_child_of: argument 1 ($dir) must have cvtype `dir'")
+            unless $dir->is_dir;
     }
     else
     {
-	$dir = $self;
+        $dir = $self;
     }
 
     return $self->ccm->query_object("is_child_of('$dir','$self')", @$keywords);
@@ -513,7 +517,7 @@ sub is_child_of
 sub has_child
 {
     my $self = shift;
-    my ($obj, $keywords) = 
+    my ($obj, $keywords) =
         validate(\@_, InstanceOf["VCS::CMSynergy::Object"], VCS::CMSynergy::_KEYWORDS());
 
     return $self->ccm->query_object("has_child('$obj','$self')", @$keywords);
@@ -526,8 +530,8 @@ sub has_child
 
   $pr = $proj->process_rule(@keywords);
 
-These are convenience methods to return 
-the I<project_grouping> (as a C<VCS::CMSynergy::Projectgrouping>) 
+These are convenience methods to return
+the I<project_grouping> (as a C<VCS::CMSynergy::Projectgrouping>)
 and I<process_rule> (as a C<VCS::CMSynergy::Object>) of the invocant project.
 
 are exactly the same as
@@ -554,6 +558,8 @@ sub project_grouping
 sub process_rule
 {
     my $self = shift;
+    # NOTE: The relation from project to process rule is still called
+    # "reconfigure_template".
     return $self->is_reconfigure_template_of(@_)->[0];
 }
 
@@ -618,11 +624,16 @@ sub top_dir
         { is_child_of => [ $self, $self ] }, @keywords)->[0];
 }
 
- 
+
 =head1 MISCELLANEOUS
 
 =head2 show_reconfigure_properties
 
+Note: This method is obsolete in Synergy 7.2 and up, as the underlying
+command C<ccm reconfigure_properties> doesn't exist anymore.
+Use L<show_object|VCS::CMSynergy::ProjectGrouping/"show">
+on the project's I<project grouping> to obtain similar information.
+
   $objects = $proj->show_reconfigure_properties($what, @keywords, \%options);
 
 Shows information about the project's reconfigure properties
@@ -643,14 +654,14 @@ shows folders that are in the project's reconfigure properties
 
 =item C<"tasks_and_folders">
 
-shows tasks and folders that are directly in the project's 
+shows tasks and folders that are directly in the project's
 reconfigure properties
 
 =item C<"all_tasks">
 
-shows all tasks that are directly or indirectly in the project's 
-reconfigure properties (indirectly means the task is in a folder 
-that is in the project's reconfigure properties) 
+shows all tasks that are directly or indirectly in the project's
+reconfigure properties (indirectly means the task is in a folder
+that is in the project's reconfigure properties)
 
 =item C<"objects">
 
@@ -659,35 +670,35 @@ in the project's reconfigure properties
 
 =back
 
-See the description of  L<VCS::CMSynergy/query_hashref> or 
-L<VCS::CMSynergy/query_object>, resp., for the meaning of 
-C<@keywords>. 
+See the description of  L<VCS::CMSynergy/query_hashref> or
+L<VCS::CMSynergy/query_object>, resp., for the meaning of
+C<@keywords>.
 
-C<show_reconfigure_properties> also accepts an optional trailing  
+C<show_reconfigure_properties> also accepts an optional trailing
 hash reference. Possible keys are:
 
 =over 4
 
 =item C<subprojects> (boolean)
 
-whether to include the reconfigure properties 
+whether to include the reconfigure properties
 of sub projects (recursively), default: false
 
 =item C<automatic> (boolean)
 
-whether automatic tasks are to be shown, default: false; 
-this option is only relevant if C<$what> is "tasks", "tasks_and_folders" 
+whether automatic tasks are to be shown, default: false;
+this option is only relevant if C<$what> is "tasks", "tasks_and_folders"
 or "all_tasks"
 
 =back
 
-Example: 
+Example:
 
   $tasks = $proj->show_reconfigure_properties(
-	     all_tasks => qw/task_synopsis completion_date/, 
-	     { subprojects => 1, automatic => 0 });
+             all_tasks => qw/task_synopsis completion_date/,
+             { subprojects => 1, automatic => 0 });
 
-=cut 
+=cut
 
 sub show_reconfigure_properties
 {
@@ -695,9 +706,7 @@ sub show_reconfigure_properties
     my $opts = @_ && ref $_[-1] eq "HASH" ? pop : {};
     my ($what, $keywords) = validate(\@_, Str, VCS::CMSynergy::_KEYWORDS());
 
-    croak(__PACKAGE__."::show_reconfigure_properties:".
-	  " argument 1 (what) must be one of tasks|folders|tasks_and_folders|all_tasks|objects")
-	unless $what =~ /^(tasks|folders|tasks_and_folders|all_tasks|objects)$/;
+    VCS::CMSynergy::_must_be_one_of($what, qw( tasks folders tasks_and_folders all_tasks objects ));
 
     my $want = VCS::CMSynergy::_want(1, $keywords);
     my $format = $VCS::CMSynergy::RS . join($VCS::CMSynergy::FS, values %$want) . $VCS::CMSynergy::FS;
@@ -707,17 +716,17 @@ sub show_reconfigure_properties
     push @cmd, "-r" if $opts->{subprojects};
 
     my ($rc, $out, $err) = $self->ccm->_ccm(
-	@cmd, -format => $format, -show => $what, $self);
+        @cmd, -format => $format, -show => $what, $self);
     return $self->ccm->set_error($err || $out) unless $rc == 0;
     # NOTE: if the reconf properties are empty, Synergy shows the string "None"
     return [ ] if $out eq "None";
 
     my @result;
-    foreach (split(/\Q${VCS::CMSynergy::RS}\E/, $out))	# split into records 
+    foreach (split(/\Q${VCS::CMSynergy::RS}\E/, $out))  # split into records
     {
-	next unless length($_);				# skip empty leading record
-	my @cols = split(/\Q${VCS::CMSynergy::FS}\E/, $_, -1);	# don't strip empty trailing fields
-	push @result, $self->ccm->_query_result($want, \@cols, 1);
+        next unless length($_);                         # skip empty leading record
+        my @cols = split(/\Q${VCS::CMSynergy::FS}\E/, $_, -1);  # don't strip empty trailing fields
+        push @result, $self->ccm->_query_result($want, \@cols, 1);
     }
     return \@result;
 }
